import logging
import time
from typing import Optional
from pydantic import BaseModel
from random import random, randint

from fastapi import FastAPI, status
from fastapi.responses import JSONResponse

log = logging.getLogger(__name__)

"""
response to mimic (at the moment we are interested only in 'dati'):
    [{
        'verbali_info': {}, 
        'permessi_info': {}, 
        'incidenti_n': 0, 
        'blacklist_n': 0, 
        'rimozioni_n': 0, 
        'prestanome': {}, 
        'targa': 'DW396EM', 
        'id': 'DW396EM_AUTOVEICOLO', 
        'tipo': 'AUTOVEICOLO', 
        'cached': False, 
        'dati': {
            'assicurato': False, 
            'revisionato': None, 
            'tipologia': 'AUTOVEICOLO', 
            'situazioneAl': '2023-11-29T00:00:00', 
            'targa': 'DW396EM', 
            'cached': False, 
            'tipoVeicolo': 'AUTOVEICOLO', 
            'veicoloAssicurato': 'COPERTURA ASSICURATIVA SCADUTA.', 
            'situazioneVeicolo': 'VEICOLO CON PERDITA DI POSSESSO IN DATA 09/05/2022 PER FURTO', 
            'dataScadenzaAssicurazione': '2022-05-16T00:00:00', 
            'dataDecorrenzaAssicurazione': '2022-03-29T00:00:00'
        }
    }]
"""


class Token(BaseModel):
    token: str
    refreshToken: str


class Login(BaseModel):
    username: str
    password: str
    realm: str
    audience: str = "ced"


class Dati(BaseModel):
    assicurato: bool
    revisionato: Optional[bool] = None
    tipologia: str
    situazioneAl: Optional[str] = None
    targa: str
    cached: bool
    tipoVeicolo: str
    veicoloAssicurato: Optional[str] = None
    situazioneVeicolo: Optional[str] = None
    dataScadenzaAssicurazione: Optional[str] = None
    dataDecorrenzaAssicurazione: Optional[str] = None


class Query(BaseModel):
    blacklist: int = 0
    prestanome: dict = {}
    dati: Dati


# ===> MAIN <===
app = FastAPI()

prefix_giano = "/rest/cruscotto"
prefix_saferplace = "/saferplace/v3/base"  # TODO: to implement


@app.get("/", status_code=200)
def hello():
    return {"msg": "Welcome to Giano!"}


@app.get(f"{prefix_giano}", response_model=Query)
def read_item(targa: str = "EN675TL", client: str = "cerbero-fake") -> Query:
    time.sleep(randint(0, 5))
    dati = {
        "assicurato": random() > 0.2,
        "revisionato": random() > 0.1,
        "tipologia": "AUTOVEICOLO",
        "targa": targa,
        "cached": False,
        "tipoVeicolo": "AUTOVEICOLO",
    }
    response = Query(dati=dati)
    log.info(response.model_dump())

    return JSONResponse(content=[response.model_dump()], status_code=status.HTTP_200_OK)


@app.post("/auth/token", response_model=Token)
def get_token(login: Login):
    if (
        login.username == "cerbero"
        and login.password == "zonacerberofake1"
        and login.realm == "cerberofake"
    ):
        return {
            "token": "c778cd06fa5a899e8fdfe58f1259b01d6",
            "refreshToken": "893a76dr4c6432e2a4dc75958a0b265ddbda13c6",
        }
    return JSONResponse(content=None, status_code=status.HTTP_403_FORBIDDEN)


@app.get(
    f"{prefix_giano}/server-error", status_code=status.HTTP_500_INTERNAL_SERVER_ERROR
)
def get_query():
    return JSONResponse(content=None, status_code=status.HTTP_500_INTERNAL_SERVER_ERROR)


@app.get(f"{prefix_giano}/timeout", status_code=200)
def get_timeout():
    time.sleep(randint(11, 20))
    return JSONResponse(content=None, status_code=status.HTTP_200_OK)


@app.get(f"{prefix_giano}/access-denyed", status_code=status.HTTP_401_UNAUTHORIZED)
def get_denyed():
    time.sleep(randint(0, 5))
    return JSONResponse(content=None, status_code=status.HTTP_401_UNAUTHORIZED)
