import logging
import os
from typing import Any

import configargparse

config = None


def get_config() -> Any:
    parser = configargparse.get_argument_parser(
        config_file_parser_class=configargparse.YAMLConfigFileParser,
        formatter_class=configargparse.ArgumentDefaultsHelpFormatter,
    )

    # parser.add_argument("--step", action="store_true", default=False, help="step mode")

    parser.add_argument(
        "--debug", action="store_true", default=False, help="debugging mode"
    )
    parser.add_argument(
        "-c", "--config", required=False, is_config_file=True, help="config file path"
    )
    parser.add_argument(
        "--local", required=False, is_config_file=True, help="local config file path"
    )

    parser.add_argument(
        "--config-save",
        required=False,
        is_write_out_config_file_arg=True,
        help="config file path",
    )

    parser.add_argument(
        "--save-context", default="/tmp/images", help="save image context path"
    )

    parser.add_argument("--socket-ip-gps", default="brain")
    parser.add_argument("--socket-port-gps", type=int, default=5012)

    parser.add_argument("--ws", default="ws://nexus:8082/ws")
    parser.add_argument("--realm", default="realm1")
    parser.add_argument("--scene", default="/samples/famila")

    parser.add_argument("--plates-truth", default="", help="path to file with plates")
    parser.add_argument("--gpx", default="", help="path to gpx track file")


    parser.add_argument(
        "--wamp-messages", default="", help="path to file with fake wamp messages"
    )
    parser.add_argument(
        "--delay", type=int, default=6, help="delay factor to slow down gpx track time"
    )
    parser.add_argument(
        "--plate-images", default="/tmp/images", help="path to upload jpg images plate"
    )
    parser.add_argument(
        "--plate-images-fake", default="", help="path to load jpg images plate"
    )

    global config
    config, _ = parser.parse_known_args()
    logging.basicConfig(
        format="%(asctime)s [%(levelname)s] %(name)s %(message)s",
        level="DEBUG" if config.debug else "INFO",
    )

    log = logging.getLogger(__name__)

    if config.save_context and not os.path.exists(config.save_context):
        os.makedirs(config.save_context)
        log.info("creating %s", config.save_context)

    if config.scene:
        config.plates_truth = os.path.join(config.scene, "truth.txt")
        config.wamp_messages = os.path.join(config.scene, "wamp.yaml")
        config.gpx = os.path.join(config.scene, "track.gpx")
        config.plate_images_fake = os.path.join(config.scene, "images")

    if config.debug:
        import jurigged
        logging.getLogger('watchdog').setLevel(logging.INFO)
        jurigged.watch(pattern="*mev_fake*.py")

    return config
