import asyncio
import logging
import time
import yaml
from pathlib import Path
import shutil

from autobahn.asyncio.wamp import ApplicationRunner, ApplicationSession
from autobahn.wamp.types import ComponentConfig

log = logging.getLogger(__name__)


class Interface(ApplicationSession):
    def __init__(self, config):
        ApplicationSession.__init__(self, ComponentConfig(config.realm))
        self.settings = config
        with open(self.settings.wamp_messages) as f:
            self._messages = yaml.safe_load(f)

        runner = ApplicationRunner(config.ws, config.realm, extra={})
        asyncio.get_event_loop().run_until_complete(runner.run(self, start_loop=False))

    async def onJoin(self, details):
        log.info("wamp session attached to realm %s", self.settings.realm)
        asyncio.create_task(self.job())

    async def job(self):
        self.publish(
            "yolodeep.start",
            plates_truth=self.settings.plates_truth,
            gpx=self.settings.gpx,
        )
        for message in self._messages:
            uri = message["uri"]
            payload = message["payload"]
            log.info(
                "publish: uri=%s delay=%.2f payload=%s", uri, message["delay"], payload
            )

            await asyncio.sleep(message["delay"] * self.settings.delay)
            if uri == "plate.new":
                payload["plate"]["timestamp"] = (
                    time.time()
                )  # dobbiamo adeguare il momento del rilievo
                key = payload["plate"]["key"]
                p = Path(self.settings.plate_images_fake)
                contexts = list(p.glob(f"{key}_*.jpg"))
                if contexts:
                    filename = str(contexts[0])
                    shutil.copy(filename, self.settings.plate_images)
                    shutil.copy(
                        filename.replace(".jpg", ".json"), self.settings.plate_images
                    )
                else:
                    log.warning("cannot find fake context image with key=%s", key)
            self.publish(uri, **payload)
        self.leave()

    def onDisconnect(self):
        asyncio.get_event_loop().stop()


def run():
    from .config import get_config

    config = get_config()
    Interface(config)
    try:
        asyncio.get_event_loop().run_forever()
    except KeyboardInterrupt:
        pass  # interface.close()
    finally:
        log.info("exit from yolodeep")


if __name__ == "__main__":
    run()
